<?php
/**
 * @see Wbcr_FactoryPages000_ImpressiveThemplate
 * @author Webcraftic <wordpress.webraftic@gmail.com>
 * @copyright (c) 01.10.2018, Webcraftic
 *
 * @since 1.0.0
 */

// Exit if accessed directly
if( !defined('ABSPATH') ) {
	exit;
}

class WWLL_SettingsPage extends WCL_Page {

	public $id = "lazy_load_settings";  // Уникальный идентификатор страницы
	public $type = "options";      // Это произвольный тип страницы

	public $page_menu_dashicon = 'dashicons-admin-settings'; // Иконка для закладки страницы, дашикон

	public $page_parent_page = "performance"; // Уникальный идентификатор родительской страницы

	/**
	 * Доступен для мультисайтов
	 * @var bool
	 */
	public $available_for_multisite = true;

	/**
	 * @param WCL_Plugin $plugin
	 */
	public function __construct($plugin)
	{
		$this->menu_title = __('Lazy Load', 'async-javascript');
		parent::__construct($plugin);
	}

	/**
	 * {@inheritDoc}
	 *
	 * @return string|void
	 * @since  1.0.0
	 */
	public function getPageTitle()
	{
		return __('Lazy Load', 'async-javascript');
	}

	/**
	 * Подключаем скрипты и стили для страницы
	 *
	 * @return void
	 * @since 1.0.0
	 * @see Wbcr_FactoryPages000_AdminPage
	 *
	 */
	public function assets($scripts, $styles)
	{
		parent::assets($scripts, $styles);
	}

	/**
	 * Передаём массив опций для создания формы с полями.
	 *
	 * @return mixed[]
	 * @since 1.0.0
	 */
	public function getPageOptions()
	{
		$options = array();

		$options[] = array(
			'type' => 'html',
			'html' => '<div class="wbcr-factory-page-group-header"><strong>' . __('Settings for LazyLoad', 'webcraftic-lazy-load') . '</strong><p>' . __('LazyLoad displays images, iframes and videos on a page only when they are visible to the user. This mechanism reduces the number of HTTP requests and improves the loading time.', 'webcraftic-lazy-load') . '</p></div>'
		);

		$options[] = array(
			'type' => 'checkbox',
			'way' => 'buttons',
			'name' => 'images',
			'title' => __('Images', 'seo-friendly-images'),
			'layout' => array('hint-type' => 'icon', 'hint-icon-color' => 'grey'),
			'default' => false
		);

		$options[] = array(
			'type' => 'checkbox',
			'way' => 'buttons',
			'name' => 'iframes',
			'title' => __('Iframes &amp; Videos', 'seo-friendly-images'),
			'layout' => array('hint-type' => 'icon', 'hint-icon-color' => 'grey'),
			'default' => false
		);

		$options[] = array(
			'type' => 'checkbox',
			'way' => 'buttons',
			'name' => 'youtube',
			'title' => __('Replace Youtube videos by thumbnail', 'seo-friendly-images'),
			'layout' => array('hint-type' => 'icon', 'hint-icon-color' => 'grey'),
			'default' => false
		);

		$formOptions = array();
		$formOptions[] = array(
			'type' => 'form-group',
			'items' => $options
		);

		return $formOptions;
	}
}